@@ -16,8 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * High-level resource cache (implementation)
+/**
+ * \file
+ * High-level resource cache implementation.
  */
 
 #include <assert.h>
@@ -29,10 +30,10 @@
 #include "utils/messages.h"
 #include "utils/ring.h"
 #include "utils/utils.h"
-#include "desktop/gui_misc.h"
+#include "netsurf/misc.h"
+#include "netsurf/content.h"
 #include "desktop/gui_internal.h"
 
-#include "content/content.h"
 #include "content/mimesniff.h"
 #include "content/hlcache.h"
 
@@ -86,7 +87,7 @@ struct hlcache_s {
 	/** Ring of retrieval contexts */
 	hlcache_retrieval_ctx *retrieval_ctx_ring;
 
-	/* statsistics */
+	/* statistics */
 	unsigned int hit_count;
 	unsigned int miss_count;
 };
@@ -146,7 +147,7 @@ static void hlcache_clean(void *ignored)
 	llcache_clean(false);
 
 	/* Re-schedule ourselves */
-	guit->browser->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
+	guit->misc->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
 }
 
 /**
@@ -178,20 +179,23 @@ static bool hlcache_type_is_acceptable(lwc_string *mime_type,
  * \param pw	Pointer to private data (hlcache_handle)
  */
 static void hlcache_content_callback(struct content *c, content_msg msg,
-		union content_msg_data data, void *pw)
+		const union content_msg_data *data, void *pw)
 {
 	hlcache_handle *handle = pw;
-	hlcache_event event;
 	nserror error = NSERROR_OK;
+	hlcache_event event = {
+		.type = msg,
+	};
 
-	event.type = msg;
-	event.data = data;
+	if (data != NULL) {
+		event.data = *data;
+	}
 
 	if (handle->cb != NULL)
 		error = handle->cb(handle, &event, handle->pw);
 
 	if (error != NSERROR_OK)
-		LOG(("Error in callback: %d", error));
+		NSLOG(netsurf, INFO, "Error in callback: %d", error);
 }
 
 /**
@@ -420,7 +424,7 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 
 	switch (event->type) {
 	case LLCACHE_EVENT_HAD_HEADERS:
-		error = mimesniff_compute_effective_type(handle, NULL, 0,
+		error = mimesniff_compute_effective_type(llcache_handle_get_header(handle, "Content-Type"), NULL, 0,
 				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
 				ctx->accepted_types == CONTENT_IMAGE,
 				&effective_type);
@@ -443,7 +447,7 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 
 		break;
 	case LLCACHE_EVENT_HAD_DATA:
-		error = mimesniff_compute_effective_type(handle,
+		error = mimesniff_compute_effective_type(llcache_handle_get_header(handle, "Content-Type"),
 				event->data.data.buf, event->data.data.len,
 				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
 				ctx->accepted_types == CONTENT_IMAGE,
@@ -462,12 +466,14 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 	case LLCACHE_EVENT_DONE:
 		/* DONE event before we could determine the effective MIME type.
 		 */
-		error = mimesniff_compute_effective_type(handle,
+		error = mimesniff_compute_effective_type(llcache_handle_get_header(handle, "Content-Type"),
 				NULL, 0, false, false, &effective_type);
-		if (error == NSERROR_OK) {
+		if (error == NSERROR_OK || error == NSERROR_NOT_FOUND) {
 			error = hlcache_migrate_ctx(ctx, effective_type);
 
-			lwc_string_unref(effective_type);
+			if (effective_type != NULL) {
+				lwc_string_unref(effective_type);
+			}
 
 			return error;
 		}
@@ -475,8 +481,8 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 		if (ctx->handle->cb != NULL) {
 			hlcache_event hlevent;
 
-			hlevent.type = CONTENT_MSG_ERROR;
-			hlevent.data.error = messages_get("BadType");
+			hlevent.type = CONTENT_MSG_ERRORCODE;
+			hlevent.data.errorcode = error;
 
 			ctx->handle->cb(ctx->handle, &hlevent, ctx->handle->pw);
 		}
@@ -535,7 +541,7 @@ hlcache_initialise(const struct hlcache_parameters *hlcache_parameters)
 	hlcache->params = *hlcache_parameters;
 
 	/* Schedule the cache cleanup */
-	guit->browser->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
+	guit->misc->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
 
 	return NSERROR_OK;
 }
@@ -544,7 +550,7 @@ hlcache_initialise(const struct hlcache_parameters *hlcache_parameters)
 void hlcache_stop(void)
 {
 	/* Remove the hlcache_clean schedule */
-	guit->browser->schedule(-1, hlcache_clean, NULL);
+	guit->misc->schedule(-1, hlcache_clean, NULL);
 }
 
 /* See hlcache.h for documentation */
@@ -560,7 +566,8 @@ void hlcache_finalise(void)
 		num_contents++;
 	}
 
-	LOG(("%d contents remain before cache drain", num_contents));
+	NSLOG(netsurf, INFO, "%d contents remain before cache drain",
+	      num_contents);
 
 	/* Drain cache */
 	do {
@@ -574,17 +581,17 @@ void hlcache_finalise(void)
 		}
 	} while (num_contents > 0 && num_contents != prev_contents);
 
-	LOG(("%d contents remaining:", num_contents));
+	NSLOG(netsurf, INFO, "%d contents remaining:", num_contents);
 	for (entry = hlcache->content_list; entry != NULL; entry = entry->next) {
 		hlcache_handle entry_handle = { entry, NULL, NULL };
 
 		if (entry->content != NULL) {
-			LOG(("	%p : %s (%d users)", entry,
-					nsurl_access(
-					hlcache_handle_get_url(&entry_handle)),
-					content_count_users(entry->content)));
+			NSLOG(netsurf, INFO, "	%p : %s (%d users)",
+			      entry,
+			      nsurl_access(hlcache_handle_get_url(&entry_handle)),
+			      content_count_users(entry->content));
 		} else {
-			LOG(("	%p", entry));
+			NSLOG(netsurf, INFO, "	%p", entry);
 		}
 	}
 
@@ -612,12 +619,13 @@ void hlcache_finalise(void)
 		hlcache->retrieval_ctx_ring = NULL;
 	}
 
-	LOG(("hit/miss %d/%d", hlcache->hit_count, hlcache->miss_count));
+	NSLOG(netsurf, INFO, "hit/miss %d/%d", hlcache->hit_count,
+	      hlcache->miss_count);
 
 	free(hlcache);
 	hlcache = NULL;
 
-	LOG(("Finalising low-level cache"));
+	NSLOG(netsurf, INFO, "Finalising low-level cache");
 	llcache_finalise();
 }
 
@@ -634,8 +642,9 @@ nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
 	assert(cb != NULL);
 
 	ctx = calloc(1, sizeof(hlcache_retrieval_ctx));
-	if (ctx == NULL)
+	if (ctx == NULL) {
 		return NSERROR_NOMEM;
+	}
 
 	ctx->handle = calloc(1, sizeof(hlcache_handle));
 	if (ctx->handle == NULL) {
@@ -665,17 +674,17 @@ nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
 			hlcache_llcache_callback, ctx,
 			&ctx->llcache);
 	if (error != NSERROR_OK) {
+		/* error retrieving handle so free context and return error */
 		free((char *) ctx->child.charset);
 		free(ctx->handle);
 		free(ctx);
-		return error;
+	} else {
+		/* successfully started fetch so add new context to list */
+		RING_INSERT(hlcache->retrieval_ctx_ring, ctx);
+
+		*result = ctx->handle;
 	}
-
-	RING_INSERT(hlcache->retrieval_ctx_ring, ctx);
-
-	*result = ctx->handle;
-
-	return NSERROR_OK;
+	return error;
 }
 
 /* See hlcache.h for documentation */
