@@ -21,12 +21,16 @@
  * NetSurf international domain name handling implementation.
  */
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <libutf8proc/utf8proc.h>
 
+#include "netsurf/inttypes.h"
+
 #include "utils/errors.h"
 #include "utils/idna.h"
 #include "utils/idna_props.h"
@@ -60,17 +64,17 @@ static nserror punycode_status_to_nserror(enum punycode_status status)
 		break;
 
 	case punycode_bad_input:
-		LOG(("Bad input"));
+		NSLOG(netsurf, INFO, "Bad input");
 		ret = NSERROR_BAD_ENCODING;
 		break;
 
 	case punycode_big_output:
-		LOG(("Output too big"));
+		NSLOG(netsurf, INFO, "Output too big");
 		ret = NSERROR_BAD_SIZE;
 		break;
 
 	case punycode_overflow:
-		LOG(("Overflow"));
+		NSLOG(netsurf, INFO, "Overflow");
 		ret = NSERROR_NOSPACE;
 		break;
 
@@ -168,7 +172,7 @@ static bool idna__contextj_rule(int32_t *label, int index, size_t len)
 			return false; /* No previous character */
 		}
 		unicode_props = utf8proc_get_property(label[index - 1]);
-		if (unicode_props->combining_class == UTF8PROC_CCC_VIRAMA) {
+		if (unicode_props->combining_class == IDNA_UNICODE_CCC_VIRAMA) {
 			return true;
 		}
 
@@ -207,7 +211,7 @@ static bool idna__contextj_rule(int32_t *label, int index, size_t len)
 			return false; /* No previous character */
 		}
 		unicode_props = utf8proc_get_property(label[index - 1]);
-		if (unicode_props->combining_class == UTF8PROC_CCC_VIRAMA) {
+		if (unicode_props->combining_class == IDNA_UNICODE_CCC_VIRAMA) {
 			return true;
 		}
 		return false;
@@ -249,7 +253,7 @@ idna__utf8_to_ucs4(const char *utf8_label,
 		return NSERROR_NOMEM;
 	}
 
-	nfc_size = utf8proc_normalise(nfc_label, nfc_size,
+	nfc_size = utf8proc_normalize_utf32(nfc_label, nfc_size,
 		UTF8PROC_STABLE | UTF8PROC_COMPOSE);
 	if (nfc_size < 0) {
 		return NSERROR_NOMEM;
@@ -434,7 +438,8 @@ static bool idna__is_valid(int32_t *label, size_t len)
 
 	/* 2. Check characters 3 and 4 are not '--'. */
 	if ((label[2] == 0x002d) && (label[3] == 0x002d)) {
-		LOG(("Check failed: characters 2 and 3 are '--'"));
+		NSLOG(netsurf, INFO,
+		      "Check failed: characters 2 and 3 are '--'");
 		return false;
 	}
 
@@ -444,7 +449,8 @@ static bool idna__is_valid(int32_t *label, size_t len)
 	if ((unicode_props->category == UTF8PROC_CATEGORY_MN) ||
 		(unicode_props->category == UTF8PROC_CATEGORY_MC) ||
 		(unicode_props->category == UTF8PROC_CATEGORY_ME)) {
-		LOG(("Check failed: character 0 is a combining mark"));
+		NSLOG(netsurf, INFO,
+		      "Check failed: character 0 is a combining mark");
 		return false;
 	}
 
@@ -453,14 +459,20 @@ static bool idna__is_valid(int32_t *label, size_t len)
 
 		/* 4. Check characters not DISALLOWED by RFC5892 */
 		if (idna_prop == IDNA_P_DISALLOWED) {
-			LOG(("Check failed: character %d (%x) is DISALLOWED", i, label[i]));
+			NSLOG(netsurf, INFO,
+			      "Check failed: character %"PRIsizet" (%x) is DISALLOWED",
+			      i,
+			      label[i]);
 			return false;
 		}
 
 		/* 5. Check CONTEXTJ characters conform to defined rules */
 		if (idna_prop == IDNA_P_CONTEXTJ) {
 			if (idna__contextj_rule(label, i, len) == false) {
-				LOG(("Check failed: character %d (%x) does not conform to CONTEXTJ rule", i, label[i]));
+				NSLOG(netsurf, INFO,
+				      "Check failed: character %"PRIsizet" (%x) does not conform to CONTEXTJ rule",
+				      i,
+				      label[i]);
 				return false;
 			}
 		}
@@ -469,14 +481,20 @@ static bool idna__is_valid(int32_t *label, size_t len)
 		/** \todo optionally we can check conformance to this rule */
 		if (idna_prop == IDNA_P_CONTEXTO) {
 			if (idna__contexto_rule(label[i]) == false) {
-				LOG(("Check failed: character %d (%x) has no CONTEXTO rule defined", i, label[i]));
+				NSLOG(netsurf, INFO,
+				      "Check failed: character %"PRIsizet" (%x) has no CONTEXTO rule defined",
+				      i,
+				      label[i]);
 				return false;
 			}
 		}
 
 		/* 7. Check characters are not UNASSIGNED */
 		if (idna_prop == IDNA_P_UNASSIGNED) {
-			LOG(("Check failed: character %d (%x) is UNASSIGNED", i, label[i]));
+			NSLOG(netsurf, INFO,
+			      "Check failed: character %"PRIsizet" (%x) is UNASSIGNED",
+			      i,
+			      label[i]);
 			return false;
 		}
 
@@ -564,7 +582,7 @@ static bool idna__verify(const char *label, size_t len)
 	}
 
 	/* Perform NFC normalisation */
-	ucs4_len = utf8proc_normalise(ucs4, u_ucs4_len,
+	ucs4_len = utf8proc_normalize_utf32(ucs4, u_ucs4_len,
 		UTF8PROC_STABLE | UTF8PROC_COMPOSE);
 	if (ucs4_len < 0) {
 		free(ucs4);
@@ -585,7 +603,8 @@ static bool idna__verify(const char *label, size_t len)
 		return true;
 	}
 
-	LOG(("Re-encoded ACE label %s does not match input", ace));
+	NSLOG(netsurf, INFO, "Re-encoded ACE label %s does not match input",
+	      ace);
 	free(ace);
 
 	return false;
@@ -638,7 +657,8 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 			/* This is already a DNS-valid ASCII string */
 			if ((idna__is_ace(host, label_len) == true) &&
 			    (idna__verify(host, label_len) == false)) {
-				LOG(("Cannot verify ACE label %s", host));
+				NSLOG(netsurf, INFO,
+				      "Cannot verify ACE label %s", host);
 				return NSERROR_BAD_URL;
 			}
 			strncpy(fqdn_p, host, label_len);
